from scrapy.exceptions import *

import warnings
warnings.warn("scrapy.core.exceptions is deprecated and will be removed in Scrapy 0.11, use scrapy.exceptions instead", \
    DeprecationWarning, stacklevel=2)
