from scrapy.contrib.exporter import JsonLinesItemExporter

import warnings
warnings.warn("Module `scrapy.contrib.exporter.jsonlines` is deprecated - use `scrapy.contrib.exporter` instead",
    DeprecationWarning, stacklevel=2)
