"""
This module contains the Scrapy Crawler once installed by calling the crawler
``install`` method, like this::

    crawler.install()

After that, you can import the (singleton) crawler like this::

    from scrapy.project import crawler

"""
