"""
This contrib implements an automatic extraction library based on an Instance
Based Learning (IBL) algorithm, as described in the following papers:

    A hierarchical approach to wrapper induction
    http://portal.acm.org/citation.cfm?id=301191 

    Extracting web data using instance based learning
    http://portal.acm.org/citation.cfm?id=1265174 

This code requires the numpy library.
"""
